@@ -2,6 +2,7 @@
 
 # Copyright (c) IPython Development Team.
 # Distributed under the terms of the Modified BSD License.
+from __future__ import annotations
 
 import logging
 import sys
@@ -24,6 +25,7 @@ from .socket import DummySocket
 
 
 class InProcessKernel(IPythonKernel):
+    """An in-process kernel."""
 
     # -------------------------------------------------------------------------
     # InProcessKernel interface
@@ -46,11 +48,11 @@ class InProcessKernel(IPythonKernel):
     # Kernel interface
     # -------------------------------------------------------------------------
 
-    shell_class = Type(allow_none=True)
+    shell_class = Type(allow_none=True)  # type:ignore[assignment]
     _underlying_iopub_socket = Instance(DummySocket, ())
     iopub_thread: IOPubThread = Instance(IOPubThread)  # type:ignore[assignment]
 
-    shell_stream = Instance(DummySocket, ())
+    shell_stream = Instance(DummySocket, ())  # type:ignore[assignment]
 
     @default("iopub_thread")
     def _default_iopub_thread(self):
@@ -64,13 +66,15 @@ class InProcessKernel(IPythonKernel):
     def _default_iopub_socket(self):
         return self.iopub_thread.background_socket
 
-    stdin_socket = Instance(DummySocket, ())  # type:ignore[assignment]
+    stdin_socket = Instance(DummySocket, ())
 
     def __init__(self, **traits):
+        """Initialize the kernel."""
         super().__init__(**traits)
 
         self._underlying_iopub_socket.observe(self._io_dispatch, names=["message_sent"])
-        self.shell.kernel = self
+        if self.shell:
+            self.shell.kernel = self
 
     async def execute_request(self, stream, ident, parent):
         """Override for temporary IO redirection."""
@@ -79,26 +83,26 @@ class InProcessKernel(IPythonKernel):
 
     def start(self):
         """Override registration of dispatchers for streams."""
-        self.shell.exit_now = False
+        if self.shell:
+            self.shell.exit_now = False
 
-    def _abort_queues(self):
+    def _abort_queues(self, subshell_id: str | None = ...):
         """The in-process kernel doesn't abort requests."""
-        pass
-
-    async def _flush_control_queue(self):
-        """No need to flush control queues for in-process"""
-        pass
 
     def _input_request(self, prompt, ident, parent, password=False):
         # Flush output before making the request.
         self.raw_input_str = None
-        sys.stderr.flush()
-        sys.stdout.flush()
+        if sys.stdout is not None:
+            sys.stdout.flush()
+        if sys.stderr is not None:
+            sys.stderr.flush()
 
         # Send the input request.
         content = json_clean(dict(prompt=prompt, password=password))
+        assert self.session is not None
         msg = self.session.msg("input_request", content, parent)
         for frontend in self.frontends:
+            assert frontend is not None
             if frontend.session.session == parent["header"]["session"]:
                 frontend.stdin_channel.call_handlers(msg)
                 break
@@ -109,7 +113,7 @@ class InProcessKernel(IPythonKernel):
         # Await a response.
         while self.raw_input_str is None:
             frontend.stdin_channel.process_events()
-        return self.raw_input_str
+        return self.raw_input_str  # type:ignore[unreachable]
 
     # -------------------------------------------------------------------------
     # Protected interface
@@ -119,17 +123,21 @@ class InProcessKernel(IPythonKernel):
     def _redirected_io(self):
         """Temporarily redirect IO to the kernel."""
         sys_stdout, sys_stderr = sys.stdout, sys.stderr
-        sys.stdout, sys.stderr = self.stdout, self.stderr
-        yield
-        sys.stdout, sys.stderr = sys_stdout, sys_stderr
+        try:
+            sys.stdout, sys.stderr = self.stdout, self.stderr
+            yield
+        finally:
+            sys.stdout, sys.stderr = sys_stdout, sys_stderr
 
     # ------ Trait change handlers --------------------------------------------
 
     def _io_dispatch(self, change):
         """Called when a message is sent to the IO socket."""
         assert self.iopub_socket.io_thread is not None
+        assert self.session is not None
         ident, msg = self.session.recv(self.iopub_socket.io_thread.socket, copy=False)
         for frontend in self.frontends:
+            assert frontend is not None
             frontend.iopub_channel.call_handlers(msg)
 
     # ------ Trait initializers -----------------------------------------------
@@ -163,6 +171,7 @@ class InProcessKernel(IPythonKernel):
 
 
 class InProcessInteractiveShell(ZMQInteractiveShell):
+    """An in-process interactive shell."""
 
     kernel: InProcessKernel = Instance(
         "ipykernel.inprocess.ipkernel.InProcessKernel", allow_none=True
@@ -184,11 +193,11 @@ class InProcessInteractiveShell(ZMQInteractiveShell):
             gui = self.kernel.gui
         return super().enable_matplotlib(gui)
 
-    def enable_pylab(self, gui=None, import_all=True, welcome_message=False):
+    def enable_pylab(self, gui=None, import_all=True):
         """Activate pylab support at runtime."""
         if not gui:
             gui = self.kernel.gui
-        return super().enable_pylab(gui, import_all, welcome_message)
+        return super().enable_pylab(gui, import_all)
 
 
 InteractiveShellABC.register(InProcessInteractiveShell)
