@@ -1,5 +1,7 @@
 """The IPython kernel implementation"""
 
+from __future__ import annotations
+
 import asyncio
 import builtins
 import gc
@@ -16,13 +18,12 @@ import comm
 from IPython.core import release
 from IPython.utils.tokenutil import line_at_cursor, token_at_cursor
 from jupyter_client.session import extract_header
-from traitlets import Any, Bool, HasTraits, Instance, List, Type, observe, observe_compat
+from traitlets import Any, Bool, HasTraits, Instance, List, Type, default, observe, observe_compat
 from zmq.eventloop.zmqstream import ZMQStream
 
 from .comm.comm import BaseComm
 from .comm.manager import CommManager
 from .compiler import XCachingCompiler
-from .debugger import Debugger, _is_debugpy_available
 from .eventloops import _use_appnope
 from .iostream import OutStream
 from .kernelbase import Kernel as KernelBase
@@ -53,7 +54,7 @@ def _create_comm(*args, **kwargs):
 
 # there can only be one comm manager in a ipykernel process
 _comm_lock = threading.Lock()
-_comm_manager: t.Optional[CommManager] = None
+_comm_manager: CommManager | None = None
 
 
 def _get_comm_manager(*args, **kwargs):
@@ -76,12 +77,18 @@ class IPythonKernel(KernelBase):
     shell = Instance("IPython.core.interactiveshell.InteractiveShellABC", allow_none=True)
     shell_class = Type(ZMQInteractiveShell)
 
+    # use fully-qualified name to ensure lazy import and prevent the issue from
+    # https://github.com/ipython/ipykernel/issues/1198
+    debugger_class = Type("ipykernel.debugger.Debugger")
+
+    compiler_class = Type(XCachingCompiler)
+
     use_experimental_completions = Bool(
         True,
         help="Set this flag to False to deactivate the use of experimental IPython completion APIs.",
     ).tag(config=True)
 
-    debugpy_stream = Instance(ZMQStream, allow_none=True) if _is_debugpy_available else None
+    debugpy_stream = Instance(ZMQStream, allow_none=True)
 
     user_module = Any()
 
@@ -91,7 +98,11 @@ class IPythonKernel(KernelBase):
         if self.shell is not None:
             self.shell.user_module = change["new"]
 
-    user_ns = Instance(dict, args=None, allow_none=True)
+    user_ns = Instance("collections.abc.Mapping", allow_none=True)
+
+    @default("user_ns")
+    def _default_user_ns(self):
+        return dict()
 
     @observe("user_ns")
     @observe_compat
@@ -109,9 +120,11 @@ class IPythonKernel(KernelBase):
         """Initialize the kernel."""
         super().__init__(**kwargs)
 
+        from .debugger import _is_debugpy_available
+
         # Initialize the Debugger
         if _is_debugpy_available:
-            self.debugger = Debugger(
+            self.debugger = self.debugger_class(
                 self.log,
                 self.debugpy_stream,
                 self._publish_debug_event,
@@ -127,7 +140,7 @@ class IPythonKernel(KernelBase):
             user_module=self.user_module,
             user_ns=self.user_ns,
             kernel=self,
-            compiler_class=XCachingCompiler,
+            compiler_class=self.compiler_class,
         )
         self.shell.displayhook.session = self.session  # type:ignore[attr-defined]
 
@@ -209,6 +222,8 @@ class IPythonKernel(KernelBase):
     }
 
     def dispatch_debugpy(self, msg):
+        from .debugger import _is_debugpy_available
+
         if _is_debugpy_available:
             # The first frame is the socket id, we can drop it
             frame = msg[1].bytes.decode("utf-8")
@@ -216,7 +231,7 @@ class IPythonKernel(KernelBase):
             self.debugger.tcp_client.receive_dap_frame(frame)
 
     @property
-    def banner(self):
+    def banner(self):  # type:ignore[override]
         if self.shell:
             return self.shell.banner
         return None
@@ -352,6 +367,11 @@ class IPythonKernel(KernelBase):
             # restore the previous sigint handler
             signal.signal(signal.SIGINT, save_sigint)
 
+    @contextmanager
+    def _dummy_context_manager(self, *args):
+        # Signals only work in main thread, so cannot use _cancel_on_sigint in subshells.
+        yield
+
     async def execute_request(self, stream, ident, parent):
         """Override for cell output - cell reconciliation."""
         parent_header = extract_header(parent)
@@ -375,13 +395,13 @@ class IPythonKernel(KernelBase):
 
         self._forward_input(allow_stdin)
 
-        reply_content: t.Dict[str, t.Any] = {}
+        reply_content: dict[str, t.Any] = {}
         if hasattr(shell, "run_cell_async") and hasattr(shell, "should_run_async"):
             run_cell = shell.run_cell_async
             should_run_async = shell.should_run_async
             accepts_params = _accepts_parameters(run_cell, ["cell_id"])
         else:
-            should_run_async = lambda cell: False  # noqa: ARG005
+            should_run_async = lambda cell: False  # noqa: ARG005, E731
             # older IPython,
             # use blocking run_cell and wrap it in coroutine
 
@@ -430,7 +450,12 @@ class IPythonKernel(KernelBase):
 
                 coro_future = asyncio.ensure_future(coro)
 
-                with self._cancel_on_sigint(coro_future):
+                cm = (
+                    self._cancel_on_sigint
+                    if threading.current_thread() == threading.main_thread()
+                    else self._dummy_context_manager
+                )
+                with cm(coro_future):
                     res = None
                     try:
                         res = await coro_future
@@ -524,6 +549,8 @@ class IPythonKernel(KernelBase):
 
     async def do_debug_request(self, msg):
         """Handle a debug request."""
+        from .debugger import _is_debugpy_available
+
         if _is_debugpy_available:
             return await self.debugger.process_request(msg)
         return None
@@ -572,7 +599,7 @@ class IPythonKernel(KernelBase):
         """Handle code inspection."""
         name = token_at_cursor(code, cursor_pos)
 
-        reply_content: t.Dict[str, t.Any] = {"status": "ok"}
+        reply_content: dict[str, t.Any] = {"status": "ok"}
         reply_content["data"] = {}
         reply_content["metadata"] = {}
         assert self.shell is not None
@@ -753,9 +780,9 @@ class IPythonKernel(KernelBase):
             for stream in [stdout, stderr]:
                 if isinstance(stream, OutStream):
                     if parent == kernel_thread_ident:
-                        stream._thread_to_parent_header[
-                            self.ident
-                        ] = kernel._new_threads_parent_header
+                        stream._thread_to_parent_header[self.ident] = (
+                            kernel._new_threads_parent_header
+                        )
                     else:
                         stream._thread_to_parent[self.ident] = parent
             _threading_Thread_run(self)
@@ -768,7 +795,7 @@ class IPythonKernel(KernelBase):
         threading.Thread.run = run_closure  # type:ignore[method-assign]
 
     def _clean_thread_parent_frames(
-        self, phase: t.Literal["start", "stop"], info: t.Dict[str, t.Any]
+        self, phase: t.Literal["start", "stop"], info: dict[str, t.Any]
     ):
         """Clean parent frames of threads which are no longer running.
         This is meant to be invoked by garbage collector callback hook.
