@@ -26,7 +26,7 @@ class ShellChannelThread(BaseThread):
         """Initialize the thread."""
         super().__init__(name=SHELL_CHANNEL_THREAD_NAME, **kwargs)
         self._manager: SubshellManager | None = None
-        self._context = context
+        self._zmq_context = context  # Avoid use of self._context
         self._shell_socket = shell_socket
 
         self.asyncio_lock = asyncio.Lock()
@@ -36,7 +36,7 @@ class ShellChannelThread(BaseThread):
         # Lazy initialisation.
         if self._manager is None:
             self._manager = SubshellManager(
-                self._context,
+                self._zmq_context,
                 self.io_loop,
                 self._shell_socket,
             )
